import gi
import pkg_resources

gi.require_version('Gtk', '3.0')
from gi.repository import Gio, Gtk
from typing import List
from ocrd_browser.window import MainWindow, AboutDialog, OpenDialog, ActionRegistry
from ocrd_browser.views import ViewManager


class OcrdBrowserApplication(Gtk.Application, ActionRegistry):
    def __init__(self):
        Gtk.Application.__init__(self, application_id='org.readmachine.ocrd-browser',
                                 flags=Gio.ApplicationFlags.HANDLES_OPEN)
        ActionRegistry.__init__(self)
        self.view_manager = ViewManager.create_from_entry_points()

    def do_startup(self):
        Gtk.Application.do_startup(self)
        self.create_simple_action('new')
        self.create_simple_action('open')
        self.create_simple_action('about')
        self.create_simple_action('quit')
        for entry_point in pkg_resources.iter_entry_points('ocrd_browser_ext'):
            (entry_point.load())(self)

    def do_activate(self):
        win = self.get_active_window()
        if not win:
            win = MainWindow(application=self)
        win.present()

    def on_about(self, _action, _param):
        about_dialog = AboutDialog(application=self, transient_for=self.get_active_window(), modal=True)
        about_dialog.present()

    def on_quit(self, _action, _param):
        self.quit()

    def on_open(self, _action, _param):
        open_dialog = OpenDialog(application=self, transient_for=self.get_active_window(), modal=True)

        response = open_dialog.run()
        if response == Gtk.ResponseType.OK:
            win = MainWindow(application=self, file=open_dialog.get_filename())
            win.present()

        open_dialog.destroy()

    def on_new(self, _action, _param):
        win = MainWindow(application=self)
        win.present()

    def do_open(self, files: List[Gio.File], file_count: int, hint: str):
        for file in files:
            win = MainWindow(application=self, file=file.get_path())
            win.present()

        return 0



#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-
import gi
import sys

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gio
from pathlib import Path

BASE_PATH = Path(__file__).absolute().parent
resources = Gio.resource_load(str(BASE_PATH / "ui.gresource"))
Gio.resources_register(resources)


def install_excepthook():
    """ Make sure we exit when an unhandled exception occurs. """
    old_hook = sys.excepthook

    def new_hook(etype, evalue, etb):
        old_hook(etype, evalue, etb)
        while Gtk.main_level():
            Gtk.main_quit()
        sys.exit()

    sys.excepthook = new_hook


def main():
    from ocrd_browser.application import OcrdBrowserApplication
    install_excepthook()
    app = OcrdBrowserApplication()
    app.run(sys.argv)


if __name__ == "__main__":
    # WHY OH WHY
    sys.path.append(str(BASE_PATH.parent))
    sys.exit(main())



import gi

try:
    gi.require_version('GtkSource', '4')
except ValueError:
    gi.require_version('GtkSource', '3.0')

from gi.repository import Gtk, GObject, GtkSource
from ocrd_models.ocrd_page import to_xml
from ocrd_browser.views import View

GObject.type_register(GtkSource.View)


# Example for extension point 'ocrd_browser_ext'
#
# 'ocrd_browser_ext': [
#    'xml = ocrd_browser.extensions.xmlview:register',
# ],
#
# def on_xml(action, param):
#     print(action)
#
# def register(application: OcrdBrowserApplication):
#     application.get_app_menu().append('XML','app.xml')
#     application.create_simple_action('xml',on_xml)


@Gtk.Template(resource_path="/org/readmachine/ocrd-browser/ui/view-xml.ui")
class ViewXml(Gtk.Box, View):
    """
    A view of the current Page-Xml with syntax highlighting via GtkSourceView
    """
    __gtype_name__ = "ViewXml"

    file_group: str = GObject.Property(type=str, default='OCR-D-IMG')

    view_action_box: Gtk.Box = Gtk.Template.Child()
    text_view: GtkSource.View = Gtk.Template.Child()
    file_group_selector: Gtk.ComboBoxText = Gtk.Template.Child()

    def __init__(self, **kwargs):
        Gtk.Box.__init__(self)
        View.__init__(self, **kwargs)
        self.buffer = self.setup_buffer()

        self.bind_property('file_group', self.file_group_selector, 'active_id', GObject.BindingFlags.BIDIRECTIONAL)
        self.connect('notify::file-group', lambda *args: self.reload())

    def setup(self):
        self.setup_file_group_selector(self.file_group_selector)
        self.setup_close_button(self.view_action_box)


    def setup_buffer(self) -> GtkSource.Buffer:
        lang_manager = GtkSource.LanguageManager()
        style_manager = GtkSource.StyleSchemeManager()
        buffer: GtkSource.Buffer = self.text_view.get_buffer()
        buffer.set_language(lang_manager.get_language('xml'))
        buffer.set_style_scheme(style_manager.get_scheme('tango'))
        return buffer

    def redraw(self):
        if self.current:
            self.buffer.set_text(to_xml(self.current.pc_gts))
            print(self.document.path(self.current.file))



import gi

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GdkPixbuf, Gio, GObject, GLib

from ocrd_browser import __version__
from ocrd_browser.model import Document
from ocrd_browser.views import ViewManager, ViewImages, View
from ocrd_browser.icon_store import LazyLoadingListStore
from ocrd_browser.image_util import cv_scale, cv_to_pixbuf

import cv2
import os


class ActionRegistry:
    def __init__(self):
        self.actions = {}

    def create_simple_action(self, name, callback=None):
        callback = callback if callback else getattr(self, 'on_' + name)
        action: Gio.SimpleAction = Gio.SimpleAction.new(name)
        action.connect("activate", callback)
        self.add_action(action)
        self.actions[name] = action
        return action


@Gtk.Template(resource_path="/org/readmachine/ocrd-browser/ui/main-window.ui")
class MainWindow(Gtk.ApplicationWindow, ActionRegistry):
    __gtype_name__ = "MainWindow"

    header_bar: Gtk.HeaderBar = Gtk.Template.Child()
    page_list_scroller: Gtk.ScrolledWindow = Gtk.Template.Child()
    panes: Gtk.Paned = Gtk.Template.Child()
    current_page_label: Gtk.Label = Gtk.Template.Child()
    view_container: Gtk.Box = Gtk.Template.Child()
    create_view_select: Gtk.ComboBoxText = Gtk.Template.Child()

    def __init__(self, file=None, **kwargs):
        Gtk.ApplicationWindow.__init__(self, **kwargs)
        ActionRegistry.__init__(self)
        self.views = []
        self.current_page_id = None
        self.create_simple_action('close')
        self.create_simple_action('goto_first')
        self.create_simple_action('go_back')
        self.create_simple_action('go_forward')
        self.create_simple_action('goto_last')
        self.create_simple_action('create_view')

        close_view_action = Gio.SimpleAction.new("close_view", GLib.Variant("s", "").get_type())
        close_view_action.connect("activate", self.on_close_view)
        self.add_action(close_view_action)

        self.document = Document.load(file)

        title = self.document.workspace.mets.unique_identifier if self.document.workspace.mets.unique_identifier else '<unnamed>'

        self.set_title(title)
        self.header_bar.set_title(title)
        self.header_bar.set_subtitle(self.document.workspace.directory)

        self.page_list = PagePreviewList(self.document)
        self.page_list_scroller.add(self.page_list)
        self.page_list.connect('page_selected', self.page_selected)

        for id_, view in self.view_manager.get_view_options().items():
            self.create_view_select.append(id_, view)
        self.add_view(ViewImages(document=self.document))

        if len(self.document.page_ids):
            self.page_selected(None, self.document.page_ids[0])

    @property
    def view_manager(self) -> ViewManager:
        return self.get_application().view_manager

    def add_view(self, view: 'View'):
        view.set_document(self.document)
        view.set_name('view_{}'.format(len(self.views)))
        view.setup()
        self.views.append(view)
        self.connect('page_activated', view.page_activated)
        self.view_container.pack_start(view, True, True, 3)

    def page_selected(self, _, page_id):
        if self.current_page_id != page_id:
            self.current_page_id = page_id
            self.emit('page_activated', page_id)

    @GObject.Signal(arg_types=(str,))
    def page_activated(self, page_id):
        index = self.document.page_ids.index(page_id)
        self.current_page_label.set_text('#{} ({}/{})'.format(page_id, index + 1, len(self.document.page_ids)))
        self.update_ui()
        pass

    def update_ui(self):
        if self.current_page_id and len(self.document.page_ids) > 0:
            index = self.document.page_ids.index(self.current_page_id)
            last_page = len(self.document.page_ids) - 1
            self.actions['goto_first'].set_enabled(index > 0)
            self.actions['go_back'].set_enabled(index > 0)
            self.actions['go_forward'].set_enabled(index < last_page)
            self.actions['goto_last'].set_enabled(index < last_page)

    def on_close(self, _a: Gio.SimpleAction, _p):
        self.destroy()

    def on_goto_first(self, _a: Gio.SimpleAction, _p):
        self.page_list.goto_index(0)

    def on_go_forward(self, _a: Gio.SimpleAction, _p):
        self.page_list.skip(1)

    def on_go_back(self, _a: Gio.SimpleAction, _p):
        self.page_list.skip(-1)

    def on_goto_last(self, _a: Gio.SimpleAction, _p):
        self.page_list.goto_index(-1)

    def on_create_view(self, _a: Gio.SimpleAction, _):
        active_id = self.create_view_select.get_active_id()
        view = self.view_manager.get_view(active_id)
        if view:
            self.add_view(view(document=self.document))

    def on_close_view(self, _action: Gio.SimpleAction, view_name: GLib.Variant):
        for view in self.views:
            if view.get_name() == view_name.get_string():
                view.destroy()


@Gtk.Template(resource_path="/org/readmachine/ocrd-browser/ui/about-dialog.ui")
class AboutDialog(Gtk.AboutDialog):
    __gtype_name__ = "AboutDialog"

    def __init__(self, **kwargs):
        Gtk.AboutDialog.__init__(self, **kwargs)
        self.set_logo(GdkPixbuf.Pixbuf.new_from_resource('/org/readmachine/ocrd-browser/icons/logo.png'))
        self.set_version(__version__)


@Gtk.Template(resource_path="/org/readmachine/ocrd-browser/ui/open-dialog.ui")
class OpenDialog(Gtk.FileChooserDialog):
    __gtype_name__ = "OpenDialog"

    def __init__(self, **kwargs):
        Gtk.FileChooserDialog.__init__(self, **kwargs)

        filter_text = Gtk.FileFilter()
        filter_text.set_name("METS files")
        filter_text.add_mime_type("text/xml")
        self.add_filter(filter_text)

        filter_any = Gtk.FileFilter()
        filter_any.set_name("Any files")
        filter_any.add_pattern("*")
        self.add_filter(filter_any)


@Gtk.Template(resource_path="/org/readmachine/ocrd-browser/ui/page-list.ui")
class PagePreviewList(Gtk.IconView):
    __gtype_name__ = "PagePreviewList"

    def __init__(self, document: Document, **kwargs):
        super().__init__(**kwargs)
        self.document = document
        self.current: Gtk.TreeIter = None
        self.model: LazyLoadingListStore = None
        self.loading_image_pixbuf: GdkPixbuf.Pixbuf = None
        self.file_lookup: dict = None
        self.setup_ui()
        self.setup_model()

    def setup_ui(self):
        theme = Gtk.IconTheme.get_default()
        self.loading_image_pixbuf = theme.load_icon_for_scale('image-loading', Gtk.IconSize.LARGE_TOOLBAR, 48, 0)
        self.set_text_column(0)
        self.set_tooltip_column(1)
        self.set_pixbuf_column(3)
        text_renderer: Gtk.CellRendererText = \
            [cell for cell in self.get_cells() if isinstance(cell, Gtk.CellRendererText)][0]
        text_renderer.ellipsize = 'middle'

    def setup_model(self):
        self.file_lookup = self.get_image_paths()
        self.model = LazyLoadingListStore(str, str, str, GdkPixbuf.Pixbuf, init_row=self.init_row,
                                          load_row=self.load_row, hash_row=self.hash_row)
        for page_id in self.document.page_ids:
            file = str(self.file_lookup[page_id])
            self.model.append((page_id, '', file, None))
        self.set_model(self.model)
        GLib.timeout_add(10, self.model.start_loading)

    def init_row(self, row: Gtk.TreeModelRow):
        row[1] = 'Loading {}'.format(row[2])
        row[3] = self.loading_image_pixbuf

    @staticmethod
    def load_row(row):
        image = cv2.imread(row[2])
        row[1] = '{} ({}x{})'.format(row[2], image.shape[1], image.shape[0])
        row[3] = cv_to_pixbuf(cv_scale(image, 100, None))
        return row

    @staticmethod
    def hash_row(row: Gtk.TreeModelRow):
        file = row[2]
        mtime = os.path.getmtime(file)
        return '{}:{}'.format(file, mtime)

    def get_image_paths(self, file_group='OCR-D-IMG'):
        images = self.document.workspace.mets.find_files(fileGrp=file_group)
        page_ids = self.document.workspace.mets.get_physical_pages(for_fileIds=[image.ID for image in images])
        file_paths = [self.document.path(image.url) for image in images]
        return dict(zip(page_ids, file_paths))

    def goto_index(self, index):
        index = index if index >= 0 else len(self.model) + index
        if 0 <= index < len(self.model):
            self.goto_path(Gtk.TreePath(index))

    def skip(self, pos):
        iterate = self.model.iter_previous if pos < 0 else self.model.iter_next
        n = iterate(self.current)
        self.goto_path(self.model.get_path(n))

    def goto_path(self, path):
        self.unselect_all()
        self.select_path(path)
        self.set_cursor(path, None, False)
        self.emit('activate_cursor_item')

    def do_item_activated(self, path: Gtk.TreePath):
        self.current = self.model.get_iter(path)
        # Trigger on_row_changed
        self.model[self.current][0] = self.model[self.current][0]
        self.emit('page_selected', self.model[path][0])

    @GObject.Signal()
    def page_selected(self, page_id: str):
        pass



import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk, GObject

import pkg_resources
from typing import Dict, Optional
from ocrd_browser.image_util import pil_to_pixbuf, pil_scale
from ocrd_browser.model import Document, Page


class View:
    def __init__(self, file_group=None, document=None):
        self.document: Document = document
        self.current: Optional[Page] = None
        self.file_group = file_group
        self.page_id = None

    def setup(self):
        pass

    def set_document(self, document: Document):
        self.document: Document = document
        self.current = None

    def page_activated(self, sender, page_id):
        self.page_id = page_id
        self.reload()

    def reload(self):
        self.current = self.document.page_for_id(self.page_id, self.file_group)
        self.redraw()

    def setup_close_button(self, view_action_box: Gtk.Box):
        close_button: Gtk.Button = Gtk.Button.new_from_icon_name('window-close',Gtk.IconSize.SMALL_TOOLBAR)
        close_button.set_name('close_button')
        close_button.set_visible(True)
        close_button.set_relief(Gtk.ReliefStyle.NONE)
        close_button.set_always_show_image(True)
        close_button.set_detailed_action_name('win.close_view("{}")'.format(self.get_name()))
        view_action_box.pack_end(close_button, False, False, 6)


    def setup_file_group_selector(self, file_group_selector):
        for group in self.document.file_groups:
            file_group_selector.append(group, group)
        file_group_selector.set_active(0)

    def redraw(self):
        pass


@Gtk.Template(resource_path="/org/readmachine/ocrd-browser/ui/view-images.ui")
class ViewImages(Gtk.Box, View):
    __gtype_name__ = "ViewImages"

    file_group: str = GObject.Property(type=str, default='OCR-D-IMG')
    page_qty: int = GObject.Property(type=int, default=1)

    image_box: Gtk.Box = Gtk.Template.Child()
    file_group_selector: Gtk.ComboBoxText = Gtk.Template.Child()
    page_qty_selector: Gtk.SpinButton = Gtk.Template.Child()
    view_action_box: Gtk.Box = Gtk.Template.Child()

    def __init__(self, **kwargs):
        Gtk.Box.__init__(self)
        View.__init__(self, **kwargs)
        self.preview_height = 10
        self.pages = []
        self.page_id = None
        self.rebuild_images()

        self.bind_property('page_qty', self.page_qty_selector, 'value', GObject.BindingFlags.BIDIRECTIONAL)
        self.connect('notify::page-qty', lambda *args: self.rebuild_images())
        self.bind_property('file_group', self.file_group_selector, 'active_id', GObject.BindingFlags.BIDIRECTIONAL)
        self.connect('notify::file-group', lambda *args: self.reload())

    def setup(self):
        self.setup_file_group_selector(self.file_group_selector)
        self.setup_close_button(self.view_action_box)


    def rebuild_images(self):
        existing_images = {child.get_name(): child for child in self.image_box.get_children()}

        for i in range(0, self.page_qty):
            name = 'image_{}'.format(i)
            if not existing_images.pop(name, None):
                image = Gtk.Image(name=name, visible=True, icon_name='gtk-missing-image', icon_size=Gtk.IconSize.DIALOG)
                self.image_box.add(image)

        for child in existing_images.values():
            child.destroy()

        self.reload()

    def page_activated(self, _sender, page_id):
        self.page_id = page_id
        self.reload()

    def reload(self):
        try:
            index = self.document.page_ids.index(self.page_id)
        except ValueError:
            return
        index = index - index % self.page_qty
        display_ids = self.document.page_ids[index:index + self.page_qty]
        self.pages = []
        for display_id in display_ids:
            self.pages.append(self.document.page_for_id(display_id, self.file_group))
        self.redraw()

    @Gtk.Template.Callback()
    def on_viewport_size_allocate(self, _sender: Gtk.Widget, rect: Gdk.Rectangle):
        if abs(self.preview_height - rect.height) > 4:
            self.preview_height = rect.height
            self.redraw()

    def redraw(self):
        if self.pages:
            for image, page in zip(self.image_box.get_children(), self.pages):
                thumbnail = pil_scale(page.image, None, self.preview_height - 10)
                image.set_from_pixbuf(pil_to_pixbuf(thumbnail))


class ViewManager:
    def __init__(self, views: Dict):
        self.views = views

    @classmethod
    def create_from_entry_points(cls) -> 'ViewManager':
        views = {}
        entry_point: pkg_resources.EntryPoint
        for entry_point in pkg_resources.iter_entry_points('ocrd_browser_view'):
            view_class = entry_point.load()
            views[entry_point.name] = view_class
        return cls(views)

    def get_view_options(self) -> Dict:
        return {id_: view_class.__name__ for id_, view_class in self.views.items()}

    def get_view(self, id_):
        return self.views[id_] if id_ in self.views else None
